$(function() {
    
    var waypointOffset = SCREEN_HEIGHT * 3;
    var currentIdx = 0;
    var colNum = 3;
    var willDisableWaypoint = false;
    var itemData = {};
    
    if (SCREEN_WIDTH >= 768) {
        colNum = 5;
    }
    else if (SCREEN_WIDTH >= 600) {
        colNum = 4;
    }
    
    var isLoading = false;
    var waypointEnabled = false;
    var cols = [];
    function refreshColumns() {
        currentIdx = 0;
        var content = $('#content');
        content.html('');
        cols = [];
        for (var i = 0; i < colNum; i++) {
            var col = document.createElement('div');
            col.setAttribute('class', 'column');
            cols.push(col);
            content.append(col);
        }
    }

    // retry btn
    $('.btn', '#retry').on('click', function() {
        loadInit({
            webId: WEB_ID,
            mode: 'retry'
        });
    });
    
    // see newest btn
    $('.label', '#newest').on('click', function() {
        willDisableWaypoint = false;
        loadInit({
            webId: WEB_ID,
            mode: 'newest'
        });
    });
    
    function hoverOn() {
        $(this).addClass('active');
    }
    function hoverOff() {
        $(this).removeClass('active');
    }
    
    // file
    $('#content')
        .on('touchstart', '.thumb', function() {
            $.data(this, 'start_time', 1 * new Date());
        })
        .on('touchmove touchcancel', '.thumb', function() {
            $.data(this, 'start_time', -1);
        })
        .on('touchend', '.thumb', function() {
            var startTime = $.data(this, 'start_time');
            var duration = ((1 * new Date()) - startTime);
            if (startTime != -1 && duration < 1000) {
                var self = this;
                if (IS_IOS) {
                    // set timeout in 90 ms b4 hoverOn(), this timeout 
                    // will be cancelled when scroll stop
                    $.data(this, 'hover_timeout', setTimeout(function() {
                         hoverOn.call(self);
                         // if hover timeout not cancelled, wait another 
                         // 410 ms b4 invoke item.click()
                         setTimeout(function() {
                             $(self).click();
                         }, 400);
                    }, 100));
                } else {
                    clearTimeout($.data(this, 'hover_timeout'));
                    hoverOn.call(this);
                    $.data(this, 'hover_timeout', setTimeout(function() { // in case of doubleclick
                        hoverOff.call(self);
                    }, 1200));
                }
            }
        })
        .on('click', '.thumb', function() {
            hoverOn.call(this);
            var self = $(this);
            var fileInfo = itemData[self.data('file-id')];
            if (fileInfo) {
                Ti.App.fireEvent('gallery_thumb:'+ SORT_TYPE +':file_click', {
                    webId: WEB_ID,
                    // fileId: self.data('file-id'),
                    // fileKind: self.data('file-kind'),
                    // fileFf: (self.data('file-ff') == '1'),
                    // fileFamily: self.data('file-family')
                    fileInfo: fileInfo
                });
            }
            hoverOff.call(this);
        });
    
    $(window).on('scroll', function() {
        if (IS_IOS) {
            $('.thumb').each(function() {
                clearTimeout($.data(this, 'hover_timeout'));
            });
        }
        if (waypointEnabled && !isLoading) {
            var self = $(this);
            var viewportHeight = (window.innerHeight || self.height());
            var panelOffset = $('#panel').offset();
            var windowTop = self.scrollTop();
            if (panelOffset.top <= (viewportHeight + windowTop + waypointOffset)) {
                loadInit({
                    webId: WEB_ID,
                    mode: 'next'
                });
            }
        }
    });

    /**
     * @param {Object} e
     *  - webId {Number}
     *  - mode {String}
     *  - override {Boolean}
     */
    function loadInit(e) {
        if (e.webId != WEB_ID) {
            return;
        }
        Ti.API.info('[M9] loadInit(), mode:'+e.mode+', isLoading:'+isLoading);
        if (isLoading && !e.override) {
            return;
        }
        waypointEnabled = false;
        if (e.override) {
            willDisableWaypoint = false;
        }
        if (!willDisableWaypoint) {
        	isLoading = true;
            if (e.mode == 'init' || e.mode == 'newest' || e.mode == 'refresh') {
                refreshColumns();
            }
            $('#loading').show();
            $('#end').hide();
            $('#newest').hide();
            $('#retry').hide();
            $('#blank').hide();
        }
        Ti.App.fireEvent('gallery_thumb:'+ SORT_TYPE +':load_ready', {
            webId: WEB_ID,
            mode: e.mode
        });
    }
    Ti.App.addEventListener('gallery_thumb:'+ SORT_TYPE +':load_init', loadInit);
    
    var density = 'ldpi';
    if (SCREEN_DENSITY >= 2) {
        density = 'xhdpi';
    }
    else if (SCREEN_DENSITY >= 1.5) {
        density = 'hdpi';
    }
    else if (SCREEN_DENSITY >= 1.0) {
        density = 'mdpi';
    }
    
    /**
     * @param {Object} e
     *  - webId {Number}
     *  - success {Boolean}
     *  - files {Array}
     *  - family {String}
     *  - hasMore {Boolean}
     *  - pageNum {Number}
     *  - sortType {Number}
     *  - willDisableWaypoint {Boolean} - disable waypoint when reach bottom
     *  - isIos {Boolean}
     *  - isFamilyFilter {Boolean}
     */
    function loadFinish(e) {
        if (e.webId != WEB_ID) {
            return;
        }
        isLoading = false;
        if (e.success) {
            willDisableWaypoint = e.willDisableWaypoint;
            var canvasSize = 20;
            var canvasSizeDp = canvasSize * SCREEN_DENSITY;
            for (var i = 0, iLen = e.files.length; i < iLen; i++) {
                var f = e.files[i];
                itemData[f.id] = f;
                var html = [];
                html.push('<div class="thumb" data-file-id="'+ f.id +'">');
                // html.push(' ');
                // html.push(' data-file-kind="'+ f.kind +'"');
                // html.push(' data-file-ff="'+ (f.ff === true ? '1' : '0') +'"');
                // html.push(' data-file-family="'+ (f.family || e.family) +'"');
                // html.push('>');
                html.push('<div class="thumb-wrap">');
                html.push('<img src="'+ f.thumb +'" onerror="this.src=\'./lib/ui/html/img/blank.gif\'"/>');
                if (f.playable) {
                    html.push('<div class="circle">');
                    html.push('<div><canvas class="p'+ e.pageNum +'" width="'+ canvasSizeDp +'" height="'+ canvasSizeDp +'"></canvas></div>');
                    html.push('</div>');
                }
                var showFFOverlay = f.ff && e.isFamilyFilter;
                if (showFFOverlay) {
                    html.push('<div class="ff"></div>');
                }
                html.push('</div>'); // .thumb-wrap
                if (showFFOverlay || (f.ff || f.kind == 'premium')) {
                    html.push('<div class="star"><i class="star-icon"></i></div>');
                }
                html.push('</div>'); // .thumb
                var div = document.createElement('div'); // turn html string to DOM object
                div.innerHTML = html.join('');
                var currentCol = currentIdx++ % colNum;
                cols[currentCol].appendChild(div.firstChild);
            }
            if (iLen == 0 && e.pageNum == 1 && !e.hasMore) {
                $('#loading').hide();
                $('#blank').show();
            } else {
                $('.circle .p'+ e.pageNum).each(function() {
                    var ctx = this.getContext('2d');
                    var radius = this.width / 2;
                    var r = radius - 1;
                    ctx.beginPath();
                    ctx.arc(radius, radius, r, 0, Math.PI * 2, false);
                    ctx.fillStyle = 'rgba(255,255,255,0.5)';
                    ctx.fill();
                    ctx.lineWidth = 1;
                    ctx.strokeStyle = '#fff';
                    ctx.stroke();
                    var im = new Image();
                    im.src = './lib/ui/html/img/'+ density +'/btn_play_small.png';
                    im.onload = function() {
                        ctx.drawImage(im, 6 * SCREEN_DENSITY, 5 * SCREEN_DENSITY, radius, radius);
                    };
                    this.style.width = canvasSize + 'px';
                    this.style.height = canvasSize + 'px';
                });
                waypointEnabled = e.hasMore;
                if (willDisableWaypoint) {
                    $('#loading').hide();
                    waypointEnabled = true;
                }
                if (e.hasMore) {
                    if (e.pageNum < 3 || $('#content .column:first .thumb').length < 12) {
                        loadInit({
                            webId: WEB_ID,
                            mode: 'next'
                        });
                        return;
                    }
                } else {
                    $('#loading').hide();
                    if (e.sortType == 2 || e.family == 'search' || e.IS_NEXT_VERSION) {
                        $('#end').show(); 
                    } else { // not sort by newest
                        $('#newest').show();
                    }
                }
            }
        } else {
            $('#loading').hide();
            $('#retry').show();
        }
    }
    Ti.App.addEventListener('gallery_thumb:'+ SORT_TYPE +':load_finish', loadFinish);
    
    /**
     * @param {Object} e
     *  - webId {Number}
     */
    function destroy(e) {
        if (e.webId != WEB_ID) {
            return;
        }
        $('#content').html('');
        Ti.App.removeEventListener('gallery_thumb:'+ SORT_TYPE +':destroy', arguments.callee);
        Ti.App.removeEventListener('gallery_thumb:'+ SORT_TYPE +':load_init', loadInit);
        Ti.App.removeEventListener('gallery_thumb:'+ SORT_TYPE +':load_finish', loadFinish);
    }
    Ti.App.addEventListener('gallery_thumb:'+ SORT_TYPE +':destroy', destroy);
    $(window).on('unload', destroy);

    Ti.App.fireEvent('gallery_thumb:'+ SORT_TYPE +':ready', {
        webId: WEB_ID
    });

});